@@ -2,6 +2,7 @@
 # -*- coding=utf-8 -*-
 
 import gettext
+import locale
 import os
 import sys
 import apt
@@ -10,23 +11,43 @@ import gi
 gi.require_version("GdkPixbuf", "2.0")
 gi.require_version("Gtk", "3.0")
 gi.require_version("XApp", "1.0")
-from gi.repository import GdkPixbuf, Gtk, XApp
+gi.require_version("PackageKitGlib", "1.0")
+from gi.repository import GdkPixbuf, Gtk, XApp, Gio, GLib
+from gi.repository import PackageKitGlib as packagekit
 from UbuntuDrivers import detect
-from aptdaemon import client
-from aptdaemon.enums import ERROR_UNKNOWN
-from aptdaemon.errors import NotAuthorizedError, TransactionFailed
-from aptdaemon.gtk3widgets import AptErrorDialog, AptProgressDialog
+import psutil
 import re
 import urllib
+import threading
 
-gettext.install("mintdrivers", "/usr/share/linuxmint/locale")
+# Used as a decorator to run things in the background
+def _async(func):
+    def wrapper(*args, **kwargs):
+        thread = threading.Thread(target=func, args=args, kwargs=kwargs)
+        thread.daemon = True
+        thread.start()
+        return thread
+    return wrapper
 
+# Used as a decorator to run things in the main loop, from another thread
+def idle(func):
+    def wrapper(*args):
+        GLib.idle_add(func, *args)
+    return wrapper
+
+APP = 'mintdrivers'
+LOCALE_DIR = "/usr/share/locale"
+locale.bindtextdomain(APP, LOCALE_DIR)
+gettext.bindtextdomain(APP, LOCALE_DIR)
+gettext.textdomain(APP)
+_ = gettext.gettext
 
 class Application():
 
     def __init__(self):
 
         self.builder = Gtk.Builder()
+        self.builder.set_translation_domain(APP)
         self.builder.add_from_file("/usr/share/linuxmint/mintdrivers/main.ui")
         self.builder.connect_signals(self)
         for o in self.builder.get_objects():
@@ -42,6 +63,9 @@ class Application():
 
         self.window_main.connect("delete_event", self.quit_application)
 
+        self.stack = self.builder.get_object("stack")
+        self.spinner = self.builder.get_object("spinner")
+
         self.button_driver_revert = Gtk.Button(label=_("Re_vert"), use_underline=True)
         self.button_driver_revert.connect("clicked", self.on_driver_changes_revert)
         self.button_driver_apply = Gtk.Button(label=_("_Apply Changes"), use_underline=True)
@@ -61,15 +85,16 @@ class Application():
         self.box_driver_action.pack_end(self.button_driver_restart, False, False, 0)
         self.box_driver_action.pack_end(self.button_driver_cancel, False, False, 0)
 
+        self.builder.get_object("error_button").connect("clicked", self.on_error_button)
+
         self.info_bar.set_no_show_all(True)
 
-        self.progress_bar = Gtk.ProgressBar()
-        self.box_driver_action.pack_end(self.progress_bar, False, False, 0)
+        self.progress_bar = Gtk.ProgressBar(valign=Gtk.Align.CENTER)
+        self.box_driver_action.pack_end(self.progress_bar, True, True, 0)
         self.progress_bar.set_visible(False)
 
         self.needs_restart = False
         self.live_mode = False
-        self.apt_client = client.AptClient()
 
         with open('/proc/cmdline') as f:
             cmdline = f.read()
@@ -84,31 +109,28 @@ class Application():
                 self.info_bar.hide()
                 self.update_cache()
 
+
+    def on_error_button(self, button):
+        self.stack.set_visible_child_name("drivers_page")
+        self.spinner.stop()
+
     def update_cache(self):
-        self.update_transaction = self.apt_client.update_cache()
-        self.update_transaction.connect("finished", self._on_cache_update_finished)
-        dia = AptProgressDialog(self.update_transaction)
-        dia.set_transient_for(self.window_main)
-        dia.set_modal(True)
-        dia.run(close_on_finished=True, show_error=True,
-                reply_handler=lambda: True,
-                error_handler=self.on_error,
-                )
+        self.stack.set_visible_child_name("refresh_page")
+        self.spinner.start()
+        task = packagekit.Task()
+        task.refresh_cache_async(True, Gio.Cancellable(), self.on_cache_update_progress, (None, ), self.on_cache_update_finished, (None, ))
 
-    def on_error(self, error):
-        if isinstance(error, NotAuthorizedError):
-                # Silently ignore auth failures
-            return
-        elif not isinstance(error, TransactionFailed):
-            # Catch internal errors of the client
-            error = TransactionFailed(ERROR_UNKNOWN, str(error))
-        dia = AptErrorDialog(error)
-        dia.run()
-        dia.hide()
-        self.apt_cache = apt.Cache()
+    def on_error(self, msg):
+        self.stack.set_visible_child_name("error_page")
+        self.spinner.stop()
+        self.builder.get_object("error_label").set_label(msg)
 
-    def _on_cache_update_finished(self, transaction, exit_state):
-        self.show_drivers()
+    def on_cache_update_progress(self, progress, ptype, data=None):
+        pass
+
+    def on_cache_update_finished(self, source, result, data=None):
+        XApp.set_window_progress(self.window_main, 0)
+        self.get_drivers_async()
 
     def quit_application(self, widget=None, event=None):
         self.clean_up_media_cdrom()
@@ -133,47 +155,59 @@ class Application():
     def check_internet_or_live_dvd(self, widget=None):
 
         try:
-            print ("Checking connectivity or live media...")
+            print ("Checking Internet connectivity...")
 
             # We either get drivers from the Internet of from the liveDVD
             # So check the connection to the Internet or scan for a liveDVD
 
             if self.check_connectivity("http://archive.ubuntu.com"):
             # We can reach the repository, everything's fine
-                print ("Internet connection detected")
+                print ("  --> Internet connection detected.")
                 self.info_bar.hide()
                 self.update_cache()
                 return
+            else:
+                print ("  --> Internet connection is missing.")
+
+
+            print ("Checking access to live media...")
 
             # We can't reach the repository, we need the installation media
             mount_point = None
             mounted_on_media_cdrom = False
 
             # Find the live media
-            try:
-                live_medias = subprocess.check_output("find /media | grep README.diskdefines", shell=True)
-                live_medias = str(live_medias, encoding='utf8').split("\n")
-                for live_media in live_medias:
-                    if ("README.diskdefines" in live_media):
-                        mount_point = live_media.replace("/README.diskdefines", "") # This is where our live DVD is mounted
+            partitions = psutil.disk_partitions()
+            for p in partitions:
+                try:
+                    if p.fstype == "iso9660":
+                        mount_point = p.mountpoint
+                        print ("Found live media: %s at %s" % (p.device, p.mountpoint))
                         # Add it to apt-cdrom
-                        os.system("sudo apt-cdrom -d \"%s\" -m add" % mount_point)
-                        if ("/media/cdrom" in mount_point):
-                            mounted_on_media_cdrom = True
-            except:
-                pass
+                        p = subprocess.Popen(["sudo", "apt-cdrom", "-d", mount_point, "-m", "add"], stderr=subprocess.PIPE)
+                        (stdout, stderr) = p.communicate()
+                        mounted_on_media_cdrom = True
+                        if 'E: ' in stderr.decode():
+                           mounted_on_media_cdrom = False
+                        break
+                except Exception as e:
+                    print ("Exception while calling apt-cdrom:", e)
 
             if mount_point is None:
                 # Not mounted..
+                print ("No live media found.")
                 self.info_bar.show()
+                self.stack.set_visible_child_name("drivers_page")
+                self.spinner.stop()
                 return
 
-            if mounted_on_media_cdrom == False:
+            if mounted_on_media_cdrom == False or not os.path.exists("/media/cdrom/README.diskdefines"):
                 # Mounted, but not in the right place...
                 print ("Binding mount %s to /media/cdrom" % mount_point)
                 self.clean_up_media_cdrom()
                 os.system("mkdir -p /media/cdrom")
-                os.system("mount --bind \"%s\" /media/cdrom" % mount_point)
+                p = subprocess.Popen(["mount", "--bind", mount_point, "/media/cdrom"], stderr=subprocess.PIPE)
+                stderr = p.communicate()
 
             # It should now be mounted to /media/cdrom, else something went wrong..
             if os.path.exists("/media/cdrom/README.diskdefines"):
@@ -182,11 +216,11 @@ class Application():
                 self.update_cache()
             else:
                 self.info_bar.show()
-        except (NotAuthorizedError, TransactionFailed) as e:
+        except Exception as e:
             print("An error occurred: {}".format(e))
             self.info_bar.show()
 
-    def on_driver_changes_progress(self, transaction, progress):
+    def on_driver_changes_progress(self, progress, ptype, data=None):
         #print(progress)
         self.button_driver_revert.set_visible(False)
         self.button_driver_apply.set_visible(False)
@@ -196,75 +230,81 @@ class Application():
         self.progress_bar.set_visible(True)
 
         self.label_driver_action.set_label(_("Applying changes..."))
-        self.progress_bar.set_fraction(progress / 100.0)
-        XApp.set_window_progress(self.window_main, progress)
+        if ptype == packagekit.ProgressType.PERCENTAGE:
+            prog_value = progress.get_property('percentage')
+            self.progress_bar.set_fraction(prog_value / 100.0)
+            XApp.set_window_progress(self.window_main, prog_value)
 
-    def on_driver_changes_finish(self, transaction, exit_state):
-        self.needs_restart = True
-        self.progress_bar.set_visible(False)
-        self.clear_changes()
-        self.apt_cache = apt.Cache()
-        self.set_driver_action_status()
-        self.update_label_and_icons_from_status()
-        self.button_driver_revert.set_visible(True)
-        self.button_driver_apply.set_visible(True)
-        self.button_driver_cancel.set_visible(False)
-        self.scrolled_window_drivers.set_sensitive(True)
+    def on_driver_changes_finish(self, source, result, installs_pending):
+        results = None
+        errors = False
+        try:
+            results = self.pk_task.generic_finish(result)
+        except Exception as e:
+            self.on_driver_changes_revert()
+            self.on_error(str(e))
+            errors = True
+        if not installs_pending:
+            self.needs_restart = (not errors)
+            self.progress_bar.set_visible(False)
+            self.clear_changes()
+            self.apt_cache = apt.Cache()
+            self.set_driver_action_status()
+            self.update_label_and_icons_from_status()
+            self.button_driver_revert.set_visible(True)
+            self.button_driver_apply.set_visible(True)
+            self.button_driver_cancel.set_visible(False)
+            self.scrolled_window_drivers.set_sensitive(True)
         XApp.set_window_progress(self.window_main, 0)
 
-    def on_driver_changes_error(self, transaction, error_code, error_details):
-        self.on_driver_changes_revert()
-        self.set_driver_action_status()
-        self.update_label_and_icons_from_status()
-        self.button_driver_revert.set_visible(True)
-        self.button_driver_apply.set_visible(True)
-        self.button_driver_cancel.set_visible(False)
-        self.scrolled_window_drivers.set_sensitive(True)
-        self.on_error(transaction.error)
-        XApp.set_window_progress(self.window_main, 0)
-
-    def on_driver_changes_cancellable_changed(self, transaction, cancellable):
-        self.button_driver_cancel.set_sensitive(cancellable)
-
     def on_driver_changes_apply(self, button):
-
+        self.pk_task = packagekit.Task()
         installs = []
         removals = []
 
         for pkg in self.driver_changes:
             if pkg.is_installed:
-                removals.append(pkg.shortname)
-                print ("Remove %s" % pkg.shortname)
+                removals.append(self.get_package_id(pkg.installed))
+                # The main NVIDIA package is only a metapackage.
+                # We need to collect its dependencies, so that
+                # we can uninstall the driver properly.
+                if 'nvidia' in pkg.shortname:
+                    for dep in self.get_dependencies(self.apt_cache, pkg.shortname, 'nvidia'):
+                        dep_pkg = self.apt_cache[dep]
+                        if dep_pkg.is_installed:
+                            removals.append(self.get_package_id(dep_pkg.installed))
             else:
-                installs.append(pkg.shortname)
-                print ("Install %s" % pkg.shortname)
-                try:
-                    for recommended in pkg.candidate.recommends:
-                        recommended_name = recommended[0].name
-                        if recommended_name in self.apt_cache:
-                            recommended_pkg = self.apt_cache[recommended_name]
-                            if recommended_pkg.is_installed:
-                                print ("Skipped %s (recommended but already installed)" % recommended_name)
-                            else:
-                                installs.append(recommended_name)
-                                print ("Install %s (recommended)" % recommended_name)
-                except:
-                    print ("A problem occurred, some recommended deps might not get installed")
+                installs.append(self.get_package_id(pkg.candidate))
 
+        self.cancellable = Gio.Cancellable()
         try:
-            self.transaction = self.apt_client.commit_packages(install=installs, remove=removals,
-                                                               reinstall=[], purge=[], upgrade=[], downgrade=[])
-            self.transaction.set_allow_unauthenticated(True)
-            self.transaction.connect("progress-changed", self.on_driver_changes_progress)
-            self.transaction.connect("cancellable-changed", self.on_driver_changes_cancellable_changed)
-            self.transaction.connect("finished", self.on_driver_changes_finish)
-            self.transaction.connect("error", self.on_driver_changes_error)
-            self.transaction.run()
+            if removals:
+                installs_pending = False
+                if installs:
+                    installs_pending = True
+                self.pk_task.remove_packages_async(removals,
+                            False,  # allow deps
+                            True,  # autoremove
+                            self.cancellable,  # cancellable
+                            self.on_driver_changes_progress,
+                            (None, ),  # progress data
+                            self.on_driver_changes_finish,  # callback ready
+                            installs_pending  # callback data
+                 )
+            if installs:
+                self.pk_task.install_packages_async(installs,
+                        self.cancellable,  # cancellable
+                        self.on_driver_changes_progress,
+                        (None, ),  # progress data
+                        self.on_driver_changes_finish,  # GAsyncReadyCallback
+                        False  # ready data
+                 )
+
             self.button_driver_revert.set_sensitive(False)
             self.button_driver_apply.set_sensitive(False)
             self.scrolled_window_drivers.set_sensitive(False)
-        except (NotAuthorizedError, TransactionFailed) as e:
-            print("Warning: install transaction not completed successfully: {}".format(e))
+        except Exception as e:
+            print("Warning: install not completed successfully: {}".format(e))
 
     def on_driver_changes_revert(self, button_revert=None):
 
@@ -283,12 +323,12 @@ class Application():
         self.button_driver_apply.set_sensitive(False)
 
     def on_driver_changes_cancel(self, button_cancel):
-        self.transaction.cancel()
+        self.cancellable.cancel()
         self.clear_changes()
 
     def on_driver_restart_clicked(self, button_restart):
         self.clean_up_media_cdrom()
-        subprocess.call(['reboot'])
+        subprocess.call(['systemctl', 'reboot'])
 
     def clear_changes(self):
         self.orig_selection = {}
@@ -329,6 +369,24 @@ class Application():
         self.button_driver_revert.set_sensitive(bool(self.driver_changes))
         self.button_driver_apply.set_sensitive(bool(self.driver_changes))
 
+
+    def get_package_id(self, ver):
+        """ Return the PackageKit package id """
+        assert isinstance(ver, apt.package.Version)
+        return "%s;%s;%s;" % (ver.package.shortname, ver.version, ver.package.architecture())
+
+    @staticmethod
+    def get_dependencies(apt_cache, package_name, pattern=None):
+        """ Get the package dependencies, which can be filtered out by a pattern """
+        dependencies = []
+        for or_group in apt_cache[package_name].candidate.dependencies:
+          for dep in or_group:
+            if dep.rawtype in ["Depends", "PreDepends"]:
+              dependencies.append(dep.name)
+        if pattern:
+          dependencies = [ x for x in dependencies if x.find(pattern) != -1 ]
+        return dependencies
+
     def gather_device_data(self, device):
         '''Get various device data used to build the GUI.
 
@@ -464,9 +522,14 @@ class Application():
                     return re.sub( ".*model name.*:", "", line, 1).strip()
         return _("Processor")
 
-    def show_drivers(self):
+    @_async
+    def get_drivers_async(self):
         self.apt_cache = apt.Cache()
         self.devices = detect.system_device_drivers()
+        self.show_drivers()
+
+    @idle
+    def show_drivers(self):
         self.driver_changes = []
         self.orig_selection = {}
         # HACK: the case where the selection is actually "Do not use"; is a little
@@ -478,72 +541,91 @@ class Application():
         self.nonfree_drivers = 0
         self.ui_building = True
         self.dynamic_device_status = {}
-        for device in sorted(self.devices.keys()):
-            (overall_status, icon, drivers) = self.gather_device_data(self.devices[device])
-            is_cpu = False
-            if "intel-microcode" in self.devices[device]['drivers'] or "amd64-microcode" in self.devices[device]['drivers']:
-                is_cpu = True
-                overall_status = _("Processor microcode")
-            brand_icon = Gtk.Image()
-            brand_icon.set_valign(Gtk.Align.START)
-            brand_icon.set_halign(Gtk.Align.CENTER)
-            brand_icon.set_from_pixbuf(self.get_device_icon(self.devices[device]))
-            driver_status = Gtk.Image()
-            driver_status.set_valign(Gtk.Align.START)
-            driver_status.set_halign(Gtk.Align.CENTER)
-            driver_status.set_from_icon_name(icon, Gtk.IconSize.MENU)
-            device_box = Gtk.Box(spacing=6, orientation=Gtk.Orientation.HORIZONTAL)
-            device_box.pack_start(brand_icon, False, False, 6)
-            device_detail = Gtk.Box(spacing=6, orientation=Gtk.Orientation.VERTICAL)
-            device_box.pack_start(device_detail, True, True, 0)
-            model_name = self.devices[device].get('model', None)
-            vendor_name = self.devices[device].get('vendor', None)
-            if is_cpu:
-                device_name = self.get_cpu_name()
-            elif vendor_name is None and model_name is None:
-                device_name = _("Unknown")
-            elif vendor_name is None:
-                device_name = model_name
-            elif model_name is None:
-                device_name = vendor_name
-            else:
-                device_name = "%s: %s" % (vendor_name, model_name)
-            widget = Gtk.Label(device_name)
-            widget.set_halign(Gtk.Align.START)
-            device_detail.pack_start(widget, True, False, 0)
-            widget = Gtk.Label("<small>{}</small>".format(overall_status))
-            widget.set_halign(Gtk.Align.START)
-            widget.set_use_markup(True)
-            device_detail.pack_start(widget, True, False, 0)
-            self.dynamic_device_status[device] = (driver_status, widget)
+        drivers_found = False
+        if len(self.devices) != 0:
+            for device in sorted(self.devices.keys()):
+                (overall_status, icon, drivers) = self.gather_device_data(self.devices[device])
+                is_cpu = False
+                if "intel-microcode" in self.devices[device]['drivers'] or "amd64-microcode" in self.devices[device]['drivers']:
+                    is_cpu = True
+                    overall_status = _("Processor microcode")
+                brand_icon = Gtk.Image()
+                brand_icon.set_valign(Gtk.Align.START)
+                brand_icon.set_halign(Gtk.Align.CENTER)
+                brand_icon.set_from_pixbuf(self.get_device_icon(self.devices[device]))
+                driver_status = Gtk.Image()
+                driver_status.set_valign(Gtk.Align.START)
+                driver_status.set_halign(Gtk.Align.CENTER)
+                driver_status.set_from_icon_name(icon, Gtk.IconSize.MENU)
+                device_box = Gtk.Box(spacing=6, orientation=Gtk.Orientation.HORIZONTAL)
+                device_box.pack_start(brand_icon, False, False, 6)
+                device_detail = Gtk.Box(spacing=6, orientation=Gtk.Orientation.VERTICAL)
+                device_box.pack_start(device_detail, True, True, 0)
+                model_name = self.devices[device].get('model', None)
+                vendor_name = self.devices[device].get('vendor', None)
+                if is_cpu:
+                    device_name = self.get_cpu_name()
+                elif vendor_name is None and model_name is None:
+                    device_name = _("Unknown")
+                elif vendor_name is None:
+                    device_name = model_name
+                elif model_name is None:
+                    device_name = vendor_name
+                else:
+                    device_name = "%s: %s" % (vendor_name, model_name)
+                if "vmware" in device_name.lower() or "virtualbox" in device_name.lower():
+                    print ("Ignoring device %s" % device_name)
+                    continue
+                if drivers["manually_installed"]:
+                    print("Ignoring device: %s (manually_installed)" % device_name)
+                    continue
+                drivers_found = True
+                widget = Gtk.Label(label=device_name)
+                widget.set_halign(Gtk.Align.START)
+                device_detail.pack_start(widget, True, False, 0)
+                widget = Gtk.Label(label="<small>{}</small>".format(overall_status))
+                widget.set_halign(Gtk.Align.START)
+                widget.set_use_markup(True)
+                device_detail.pack_start(widget, True, False, 0)
+                self.dynamic_device_status[device] = (driver_status, widget)
 
-            option_group = None
-            # define the order of introspection
-            for section in ('recommended', 'alternative', 'manually_installed', 'no_driver'):
-                for driver in sorted(drivers[section], key=lambda x: self.sort_string(drivers[section], x)):
-                    radio_button = Gtk.RadioButton.new(None)
-                    label = Gtk.Label()
-                    label.set_markup(drivers[section][driver]['description'])
-                    radio_button.add(label)
-                    if option_group:
-                        radio_button.join_group(option_group)
-                    else:
-                        option_group = radio_button
-                    device_detail.pack_start(radio_button, True, False, 0)
-                    radio_button.set_active(drivers[section][driver]['selected'])
+                option_group = None
+                # define the order of introspection
+                for section in ('recommended', 'alternative', 'manually_installed', 'no_driver'):
+                    for driver in sorted(drivers[section], key=lambda x: self.sort_string(drivers[section], x)):
+                        if str(driver).startswith("nvidia-driver") and str(driver).endswith("-server"):
+                            print("Ignoring server NVIDIA driver: ", driver)
+                            continue
+                        radio_button = Gtk.RadioButton.new(None)
+                        label = Gtk.Label()
+                        label.set_markup(drivers[section][driver]['description'])
+                        radio_button.add(label)
+                        if option_group:
+                            radio_button.join_group(option_group)
+                        else:
+                            option_group = radio_button
+                        device_detail.pack_start(radio_button, True, False, 0)
+                        radio_button.set_active(drivers[section][driver]['selected'])
 
-                    if section == 'no_driver':
-                        self.no_drv.append(radio_button)
-                        if is_cpu:
-                            label.set_markup(_("Do not update the CPU microcode"))
-                    if section in ('manually_install', 'no_driver') or ('builtin' in drivers[section][driver] and drivers[section][driver]['builtin']):
-                        radio_button.connect("toggled", self.on_driver_selection_changed, device)
-                    else:
-                        radio_button.connect("toggled", self.on_driver_selection_changed, device, driver)
-                    if drivers['manually_installed'] and section != 'manually_installed' and "firmware" not in str(driver):
-                        radio_button.set_sensitive(False)
+                        if section == 'no_driver':
+                            self.no_drv.append(radio_button)
+                            if is_cpu:
+                                label.set_markup(_("Do not update the CPU microcode"))
+                        if section in ('manually_install', 'no_driver') or ('builtin' in drivers[section][driver] and drivers[section][driver]['builtin']):
+                            radio_button.connect("toggled", self.on_driver_selection_changed, device)
+                        else:
+                            radio_button.connect("toggled", self.on_driver_selection_changed, device, driver)
+                        if drivers['manually_installed'] and section != 'manually_installed' and "firmware" not in str(driver):
+                            radio_button.set_sensitive(False)
 
-            self.box_driver_detail.pack_start(device_box, False, False, 6)
+                self.box_driver_detail.pack_start(device_box, False, False, 6)
+                self.stack.set_visible_child_name("drivers_page")
+
+        if not drivers_found:
+            self.stack.set_visible_child_name("no_drivers_page")
+            print("Your computer does not need any additional drivers")
+
+        self.spinner.stop()
 
         self.ui_building = False
         self.box_driver_detail.show_all()
@@ -561,11 +643,11 @@ class Application():
         '''Update the current label and icon, computing the new device status'''
 
         for device in self.devices:
-            (overall_status, icon, drivers) = self.gather_device_data(self.devices[device])
-            (driver_status, widget) = self.dynamic_device_status[device]
-
-            driver_status.set_from_icon_name(icon, Gtk.IconSize.MENU)
-            widget.set_label("<small>{}</small>".format(overall_status))
+            if device in self.dynamic_device_status.keys():
+                (overall_status, icon, drivers) = self.gather_device_data(self.devices[device])
+                (driver_status, widget) = self.dynamic_device_status[device]
+                driver_status.set_from_icon_name(icon, Gtk.IconSize.MENU)
+                widget.set_label("<small>{}</small>".format(overall_status))
 
     def set_driver_action_status(self):
         # Update the label in case we end up having some kind of proprietary driver in use.
@@ -592,8 +674,5 @@ class Application():
             self.label_driver_action.set_label(_("No proprietary drivers are in use."))
 
 if __name__ == "__main__":
-    if os.getuid() != 0:
-        os.execvp("gksu", ("", " ".join(sys.argv)))
-    else:
-        Application()
-        Gtk.main()
+    Application()
+    Gtk.main()
